import { createEvent, createStore, sample, attach, is } from 'effector';
export function interval(_ref) {
  var {
    timeout,
    start,
    stop,
    leading = false,
    trailing = false
  } = _ref;
  var setup = start !== null && start !== void 0 ? start : createEvent({
    name: "setup",
    sid: "-essz1d"
  });
  var teardown = stop !== null && stop !== void 0 ? stop : createEvent({
    name: "teardown",
    sid: "sv95oj"
  });
  var tick = createEvent({
    name: "tick",
    sid: "-r3kxtw"
  });
  var $isRunning = createStore(false, {
    name: "$isRunning",
    sid: "-jo5dgs"
  });
  var $timeout = toStoreNumber(timeout);
  var $notRunning = $isRunning.map(running => !running, {
    skipVoid: false
  });
  var saveTimeout = createEvent({
    name: "saveTimeout",
    sid: "e3gpcx"
  });
  var $timeoutId = createStore(null, {
    name: "$timeoutId",
    sid: "3759wd"
  }).on(saveTimeout, (_, _ref2) => {
    var {
      timeoutId
    } = _ref2;
    return timeoutId;
  });
  // eslint-disable-next-line @typescript-eslint/no-empty-function
  var $rejecter = createStore(() => {}, {
    name: "$rejecter",
    sid: "shnzo1"
  }).on(saveTimeout, (_, _ref3) => {
    var {
      reject
    } = _ref3;
    return reject;
  });
  var timeoutFx = attach({
    and: {
      source: {
        timeout: $timeout,
        running: $isRunning
      },
      effect: _ref4 => {
        var {
          timeout,
          running
        } = _ref4;
        if (!running) {
          return Promise.reject();
        }
        return new Promise((resolve, reject) => {
          var timeoutId = setTimeout(resolve, timeout);
          saveTimeout({
            timeoutId,
            reject
          });
        });
      }
    },
    or: {
      name: "timeoutFx",
      sid: "n7tmmy"
    }
  });
  var cleanupFx = attach({
    and: {
      source: {
        timeoutId: $timeoutId,
        rejecter: $rejecter
      },
      effect: _ref5 => {
        var {
          timeoutId,
          rejecter
        } = _ref5;
        rejecter();
        if (timeoutId) clearTimeout(timeoutId);
      }
    },
    or: {
      name: "cleanupFx",
      sid: "-h9rkrq"
    }
  });
  sample({
    and: [{
      clock: setup,
      source: $timeout,
      filter: $notRunning,
      target: timeoutFx
    }],
    or: {
      sid: "-v8qzb1"
    }
  });
  if (leading) {
    var onReady = sample({
      and: [{
        clock: setup,
        filter: $notRunning
      }],
      or: {
        name: "onReady",
        sid: "13etu6"
      }
    });
    sample({
      and: [{
        clock: onReady,
        target: tick
      }],
      or: {
        sid: "-us95gh"
      }
    });
  }
  sample({
    and: [{
      clock: setup,
      fn: () => true,
      target: $isRunning
    }],
    or: {
      sid: "-uqlrq4"
    }
  });
  sample({
    and: [{
      clock: timeoutFx.done,
      source: $timeout,
      filter: $isRunning,
      target: timeoutFx
    }],
    or: {
      sid: "-ubrbpd"
    }
  });
  sample({
    and: [{
      clock: timeoutFx.done,
      filter: $isRunning,
      target: tick.prepend(() => {
        /* to be sure, nothing passed to tick */
      })
    }],
    or: {
      sid: "6k91qi"
    }
  });
  if (trailing) {
    sample({
      and: [{
        clock: teardown,
        target: tick
      }],
      or: {
        sid: "6p774h"
      }
    });
  }
  $isRunning.on(teardown, () => false);
  sample({
    and: [{
      clock: teardown,
      target: cleanupFx
    }],
    or: {
      sid: "7558a8"
    }
  });
  return {
    tick,
    isRunning: $isRunning,
    '@@trigger': () => ({
      setup,
      teardown,
      fired: tick
    })
  };
}
function toStoreNumber(value) {
  if (is.store(value, {
    sid: "81l3dz"
  })) return value;
  if (typeof value === 'number') {
    return createStore(value, {
      and: {
        name: '$timeout'
      },
      sid: "-xt76rp"
    });
  }
  throw new TypeError("timeout parameter in interval method should be number or Store. \"".concat(typeof value, "\" was passed"));
}

/**
 * @see {@link https://withease.pages.dev/protocols/trigger.html}
 */